import os
import uuid
from io import BytesIO

import boto3
import pytest
from moto import mock_s3

from renameit.core.models import FileObject
from renameit.file_systems.aws import FileSystemAwsS3

KEYS = ["file1.txt", "folder/file2.txt", "folder/subfolder/file3.txt"]

RUN_ID = uuid.uuid4().hex
SOURCE_BUCKET = f"renameit-source-bucket-{RUN_ID}"
TARGET_BUCKET = f"renameit-target-bucket-{RUN_ID}"


@pytest.fixture(scope="class", autouse=True)
def aws_credentials():
    """Mocked AWS Credentials for moto."""
    os.environ["AWS_ACCESS_KEY_ID"] = "testing"
    os.environ["AWS_SECRET_ACCESS_KEY"] = "testing"
    os.environ["AWS_SECURITY_TOKEN"] = "testing"
    os.environ["AWS_SESSION_TOKEN"] = "testing"
    os.environ["AWS_DEFAULT_REGION"] = "us-east-2"


@pytest.fixture(autouse=True, scope="class")
def mock_s3_context():
    """This fixture creates the moto mock context for s3"""
    mock_s3_context = mock_s3()
    mock_s3_context.start()
    yield mock_s3_context
    mock_s3_context.stop()


@pytest.fixture(scope="class")
def s3():
    """This fixture creates a mock s3 resource"""
    return boto3.resource("s3")


@pytest.fixture(scope="class")
def setup(s3):
    # Create source bucket
    s3.create_bucket(
        Bucket=SOURCE_BUCKET,
        CreateBucketConfiguration={"LocationConstraint": "us-east-2"},
    )

    # Create target bucket
    s3.create_bucket(
        Bucket=TARGET_BUCKET,
        CreateBucketConfiguration={"LocationConstraint": "us-east-2"},
    )

    # Populate source bucket
    for key in KEYS:
        with BytesIO(b"Sample data") as bytes_io:
            s3.Object(SOURCE_BUCKET, key).upload_fileobj(bytes_io)


@pytest.fixture()
def filesystem(request):
    """
    Initializes a filesystem with the right params within the context
    of the mock s3 resource.
    """

    # Fixed params as there is no need for these to be variable
    params = {"source_bucket": SOURCE_BUCKET, "target_bucket": TARGET_BUCKET}

    # This allows tests functions or other fixtures to parametarize the
    # creation of filesystem instance.
    # Input params are parsed here if parametrization was defined.
    for arg in {
        "source_dir",
        "target_dir",
        "recursive",
        "aws_access_key_id",
        "aws_secret_access_key",
    }:
        params[arg] = getattr(request, "param", {}).get(arg, None)

    return FileSystemAwsS3(**params)


@pytest.mark.usefixtures("setup")
class TestFileSystemAwsS3:
    @pytest.mark.parametrize(
        ["filesystem", "expected_list_result"],
        [
            ({"recursive": False}, ["file1.txt"]),
            ({"recursive": True}, KEYS),
            ({"source_dir": "folder", "recursive": False}, ["file2.txt"]),
            ({"source_dir": "folder", "recursive": True}, ["file2.txt", "subfolder/file3.txt"]),
            (
                {"aws_access_key_id": "test_key", "aws_secret_access_key": "test_secret"},
                ["file1.txt"],
            ),
        ],
        indirect=["filesystem"],
    )
    def test_list_objects(self, filesystem, expected_list_result):
        file_objects = filesystem.list_objects()
        assert [obj.path for obj in file_objects] == expected_list_result

    @pytest.mark.parametrize(
        ["source_obj", "target_obj"],
        [
            (FileObject("file1.txt"), FileObject("copy_of_file1.txt")),
            (FileObject("folder/file2.txt"), FileObject("folder/copy_of_file2.txt")),
        ],
    )
    def test_copy_object(self, s3, filesystem, source_obj, target_obj):
        filesystem.copy_object(source_obj, target_obj)

        # Access the mock `s3` resource in order to load
        # the original and copied keys and make sure they exist.

        # We expect the `source_obj` to exist in
        # s3://SOURCE_BUCKET/source_obj.path
        s3.Bucket(SOURCE_BUCKET).Object(source_obj.path).load()

        # We expect the `target_obj` to exist in
        # s3://TARGET_BUCKET/target_obj
        s3.Bucket(TARGET_BUCKET).Object(target_obj.path).load()



import logging
from typing import Iterable, Union

from azure.identity import ClientSecretCredential
from azure.storage.filedatalake import DataLakeServiceClient

from renameit.core.models import FileObject
from renameit.core.util import add_slash
from renameit.file_systems.interfaces import IFileSystem


# FileSystemAzureDatalakeStorage # noqa
class FileSystemAzureDatalakeStorage(IFileSystem):
    """Manage files on azure datalake storage gen2.

    Supports multiple authentication methods:

      - Storage account name and access keys
      - Storage account connection strings
      - Service principal with RBAC assignments

    Args:
        source_container (str): Name of storage account container with the original files.
        target_container (str): Name of storage account container that should contain new files.
            Defaults to be the same as source_container and it will be created automatically if it
            doesn't exist.
        source_dir (str, optional): Directory path from where to start looking for files.
            If not provided defaults to source_container root.
        target_dir (str, optional): Directory path to where should the new files be written.
            If not provided defaults to target_container root.
        secure_transfer (bool, optional): Whether to use https or not. Uses https by default.
        recursive (bool, optional): Whether Decides whether to look for files recursively.
            Defaults to True.
        connection_string (str, optional): The storage account connection string.
        storage_account_name (str, optional): Storage account name.
        storage_account_key (str, optional): Storage account access key.
        tenant_id (str, optional): Directory (tenant) ID of the service principal (app).
        client_id (str, optional): Application (client) ID of the service principal (app).
        client_secret (str, optional): The app password that it uses to authenticate itself.
        account_url (str, optional): Provided if the storage account was behind a custom
            domain name. Defaults to <protocol>://<storage_account_name>.dfs.core.windows.net.
    """

    name = "azure_datalake_storage"

    def __init__(
        self,
        source_container: str,
        target_container: str = None,
        source_dir: str = None,
        target_dir: str = None,
        secure_transfer: bool = True,
        recursive: bool = True,
        connection_string: str = None,
        storage_account_name: str = None,
        storage_account_key: str = None,
        tenant_id: str = None,
        client_id: str = None,
        client_secret: str = None,
        account_url: str = None,
    ):

        super().__init__()

        self.source_container = source_container
        self.target_container = target_container or source_container
        self.source_dir = source_dir
        self.target_dir = target_dir
        self.protocol = "https" if secure_transfer else "http"
        self.recursive = recursive

        self.connection_string = connection_string
        self.storage_account_name = storage_account_name
        self.storage_account_key = storage_account_key
        self.tenant_id = tenant_id
        self.client_id = client_id
        self.client_secret = client_secret

        self.account_url = (
            account_url or f"{self.protocol}://{self.storage_account_name}.dfs.core.windows.net"
        )

        self.service_client = self._get_service_client()
        self.source_file_system_client = self.service_client.get_file_system_client(
            file_system=self.source_container
        )
        self.target_file_system_client = self.service_client.get_file_system_client(
            file_system=self.target_container
        )

        if not self.target_file_system_client.exists():
            self.target_file_system_client.create_file_system()

    def _get_service_client(self) -> DataLakeServiceClient:
        if self.connection_string:
            return DataLakeServiceClient.from_connection_string(self.connection_string)
        elif self.storage_account_name:

            credential: Union[ClientSecretCredential, str]

            if self.storage_account_key:
                credential = self.storage_account_key
            elif self.tenant_id and self.client_id and self.client_secret:
                credential = ClientSecretCredential(
                    tenant_id=self.tenant_id,
                    client_id=self.client_id,
                    client_secret=self.client_secret,
                )
            else:
                raise ValueError(
                    "Not enough credentials provided in order to connect to azure storage service."
                )

            return DataLakeServiceClient(account_url=self.account_url, credential=credential)

        else:
            raise ValueError("Either 'connection_string' or 'storage_account_name' is required.")

    def list_objects(self) -> Iterable[FileObject]:
        paths = self.source_file_system_client.get_paths(
            path=self.source_dir, recursive=self.recursive
        )

        for path in paths:
            if not path.is_directory:
                yield FileObject(
                    path=path.name.removeprefix(add_slash(self.source_dir)),
                    modified_date=path.last_modified,
                )

    def rename_object(self, source_obj: FileObject, target_obj: FileObject) -> None:
        logging.info(
            "Renaming: "
            + f"{self.account_url}/{self.source_container}/"
            + f"{add_slash(self.source_dir)}{source_obj.path}"
            + " to "
            + f"{self.account_url}/{self.target_container}/"
            + f"{add_slash(self.target_dir)}{target_obj.path}"
        )

        full_target_path = add_slash(self.target_dir) + target_obj.path

        self.target_file_system_client.create_file(full_target_path)

        self.source_file_system_client.get_file_client(
            file_path=add_slash(self.source_dir) + source_obj.path
        ).rename_file(new_name=self.target_container + "/" + full_target_path)



import os
import tempfile

import pytest


@pytest.fixture
def cleandir():
    with tempfile.TemporaryDirectory() as newpath:
        old_cwd = os.getcwd()
        os.chdir(newpath)
        yield
        os.chdir(old_cwd)



"""Main module to expose internal functions as command lines."""

from typing import Optional

import fire

from .config import ProjectConfig
from .job_runners import JobRunnerStandard


def renameit(config_path: Optional[str] = None, config_dir: Optional[str] = None):
    """This is the main API that is used to trigger renaming jobs.

    Args:
        config_path: File path to configs overriding the default path. Defaults to None.
        config_dir: Dir path to other config files if needed. Defaults to None.
    """
    config = ProjectConfig.create_new(config_path=config_path, config_dir=config_dir)

    job_runner = JobRunnerStandard(jobs=config.get_jobs())

    job_runner.execute()


def main():
    """Wrapper function for console_script entry_points"""
    fire.Fire(renameit)


# For debugging purposes
if __name__ == "__main__":
    main()



"""Module for general toolkit functionalities"""


import importlib
import inspect
import pkgutil
from typing import Any, Dict, List


def get_implementations(package, interface: Any) -> Dict[str, Any]:
    """This function provides a way to discover class implementations of `interface`
    in a specific package `package`.

    Example:

    ```python
    from app import package_1

    implementations = get_implementations(package_1, interface)
    ```

    Args:
        package: Package to iterate, pass an imported module.
        interface (Any): Interface to discover implementations for.

    Returns:
        Dict: Pairs of class's name attribute mapped to the class itself.
    """
    class_implementations: List = []
    for module_info in pkgutil.iter_modules(package.__path__, package.__name__ + "."):
        module = importlib.import_module(module_info.name)
        classes_found = inspect.getmembers(
            module,
            lambda member: inspect.isclass(member)
            and issubclass(member, interface)
            and member != interface,
        )
        class_implementations = class_implementations + classes_found

    return {c[1].name: c[1] for c in class_implementations}


def add_slash(value: str = None):
    if value is None:
        return ""
    elif value.endswith("/"):
        return value
    else:
        return value + "/"



"""
Config classes, simply extending the pydantic BaseModel are defined here to provide
a contract for users to abide by. Schema definition, validation, serialization and
deserialization are all handled within these classes.

We can expose real object instances from within these contracts/configs that can be used
later by the app logic in other modules and functions.
"""
from __future__ import annotations

import logging
from typing import Any, Dict, List, Literal, Union

from pydantic import BaseModel, Field, validator

from renameit.file_systems.interfaces import IFileSystem, file_systems_map
from renameit.handlers.interfaces import IHandler, handlers_map

from ..job_runners import Job
from .parse import load_config


class ProjectConfig(BaseModel):
    """UserProjectConfig."""

    version: str = "0.1"
    jobs: List[JobConfig] = Field(default_factory=list)

    @classmethod
    def create_new(cls, config_path, config_dir=None) -> "ProjectConfig":
        return cls(**load_config(user_file_path=config_path, user_dir_path=config_dir))

    def get_jobs(self) -> List[Job]:
        """Get a list of Job objects from their corresponding list of JobConfig.

        Returns:
            List[Job]: List of Job objects
        """
        return [job for job_config in self.jobs if (job := job_config.get_job()) is not None]


class JobConfig(BaseModel):
    """JobConfig."""

    name: str
    operation: Literal["rename", "copy"]
    file_system_config: FileSystemConfig = Field(alias="file_system")
    rename_handler_config: RenameHandlerConfig = Field(alias="rename_handler")

    def get_job(self) -> Union[Job, None]:
        try:
            return Job(
                name=self.name,
                file_system=self.file_system_config.get_file_system(),
                rename_handler=self.rename_handler_config.get_handler(),
                operation=f"{self.operation}_object",
            )
        except (NotImplementedError, TypeError) as e:
            logging.warning(f"Skipping job {self.name}: {str(e)}")

            if isinstance(e, TypeError):
                logging.exception(str(e))

            return None


class FileSystemConfig(BaseModel):
    """FileSystemConfig."""

    type: str
    args: Dict[str, Any]

    @validator("type")
    def type_validation(cls, value) -> str:
        if value not in file_systems_map.keys():
            raise ValueError(
                f"Unknow type `{value}`, only allowed: {', '.join(file_systems_map.keys())}"
            )
        return value

    def get_file_system(self) -> IFileSystem:
        return file_systems_map[self.type](**self.args)  # type: ignore


class RenameHandlerConfig(BaseModel):
    """RenameHandlerConfig."""

    type: str
    args: Dict[str, Any]

    @validator("type")
    def type_validation(cls, value) -> str:
        if value not in handlers_map.keys():
            raise ValueError(
                f"Unknow type `{value}`, only allowed: {', '.join(handlers_map.keys())}"
            )
        return value

    def get_handler(self) -> IHandler:
        return handlers_map[self.type](**self.args)  # type: ignore


ProjectConfig.update_forward_refs()
JobConfig.update_forward_refs()



import logging
import pathlib
import shutil
from datetime import datetime
from functools import wraps
from typing import Iterable

from renameit.core.models import FileObject
from renameit.file_systems.interfaces import IFileSystem


def _create_target_dirs(_fn):
    """Decorator to use on FileSystemLocal methods in order to create
    target directories if needed."""

    @wraps(_fn)
    def wrapper(self, *args, **kwargs):
        if "target_obj" in kwargs:
            (self.target_dir / kwargs["target_obj"].path).parent.mkdir(parents=True, exist_ok=True)
        return _fn(self, *args, **kwargs)

    return wrapper


# FileSystemLocal # noqa
class FileSystemLocal(IFileSystem):
    """Manage files on the local operating system.

    Args:
        source_dir (str): Source directory to iterate files from.
        target_dir (str): Target directory to copy/rename files to.
        recursive (bool, optional): Decides if listing source_dir should be recursive or not.
    """

    name = "local"

    def __init__(self, source_dir: str, target_dir: str, recursive: bool = False) -> None:

        super().__init__()

        self.source_dir = pathlib.Path(source_dir)
        self.target_dir = pathlib.Path(target_dir)
        self.recursive = recursive

    def list_objects(self) -> Iterable[FileObject]:
        if self.recursive:
            glob_method = self.source_dir.rglob
        else:
            glob_method = self.source_dir.glob

        for path in glob_method("*"):
            if path.is_file():
                yield FileObject(
                    path=str(path.relative_to(self.source_dir)),
                    modified_date=datetime.fromtimestamp((path.stat().st_mtime)),
                )

    @_create_target_dirs
    def rename_object(self, source_obj: FileObject, target_obj: FileObject) -> None:
        logging.info(
            "Renaming: "
            + str(self.source_dir / source_obj.path)
            + " to "
            + str(self.target_dir / target_obj.path)
        )

        (self.source_dir / source_obj.path).rename(self.target_dir / target_obj.path)

    @_create_target_dirs
    def copy_object(self, source_obj: FileObject, target_obj: FileObject) -> None:
        logging.info(
            "Copying: "
            + str(self.source_dir / source_obj.path)
            + " to "
            + str(self.target_dir / target_obj.path)
        )

        shutil.copy(self.source_dir / source_obj.path, self.target_dir / target_obj.path)



"""BasicHandler"""

import pathlib

from renameit.core.models import FileObject
from renameit.handlers.interfaces import IHandler


# BasicHandler # noqa
class BasicHandler(IHandler):
    """Basic file name manipulation.

    Args:
        method (str): Specify how to modify the file name. Accepted values are
            (`add_prefix, add_suffix, remove_prefix, remove_suffix`).
        value (str): Value to add or remove from the file name.
    """

    name = "basic"

    def __init__(self, method: str, value: str):

        super().__init__()

        if method not in ["add_prefix", "add_suffix", "remove_prefix", "remove_suffix"]:
            raise ValueError(f"method: '{method}' is not supported.")

        self.method = method
        self.value = value

    def get_new_name(self, file_obj: FileObject) -> FileObject:
        path = pathlib.Path(file_obj.path)
        return FileObject(
            path=str(
                {
                    "add_prefix": path.with_name(self.value + path.name),
                    "add_suffix": path.with_name(path.stem + self.value + path.suffix),
                    "remove_prefix": path.with_name(path.name.removeprefix(self.value)),
                    "remove_suffix": path.with_name(
                        path.stem.removesuffix(self.value) + path.suffix
                    ),
                }[self.method]
            ),
            modified_date=file_obj.modified_date,
        )



# pylint: disable=logging-fstring-interpolation
"""This module contains job runner classes that will run the renaming jobs

The module contains the following:

- `Job` - Class representing renaming job.
- `JobRunnerStandard` - Implementation for standard job runner.
"""


import logging
from typing import List

from ..file_systems.interfaces import IFileSystem
from ..handlers.interfaces import IHandler


class Job:
    """The job class that contains the processing details. Providing the central point
    for file systems and handlers interactions.

    Args:
        name (str): Name of the job.
        file_system (IFileSystem): Which file system to use.
        rename_handler (IHandler): Which renaming handler to use.
        operation (str): What operation to do on the matched files,
                        accepted values are (rename, copy) case sensitive.
    """

    def __init__(
        self,
        name: str,
        file_system: IFileSystem,
        rename_handler: IHandler,
        operation: str,
    ):

        if (
            file_system.__class__.copy_object == IFileSystem.copy_object
            and operation == "copy_object"
        ):
            raise NotImplementedError(
                f"FileSystem type `{file_system.name}` does not support copying objects."
            )
        if (
            file_system.__class__.rename_object == IFileSystem.rename_object
            and operation == "rename_object"
        ):
            raise NotImplementedError(
                f"FileSystem type `{file_system.name}` does not support renaming objects."
            )

        self.name = name
        self.file_system = file_system
        self.rename_handler = rename_handler
        self.operation = operation

    def run(self):
        """A job needs to run at some point."""

        logging.info(f"Running job {self.name}")

        for file_obj in self.file_system.list_objects():
            new_file_obj = self.rename_handler.get_new_name(file_obj=file_obj)

            if file_obj.path != new_file_obj.path:
                getattr(self.file_system, self.operation)(
                    source_obj=file_obj,
                    target_obj=self.rename_handler.get_new_name(file_obj=file_obj),
                )


class IJobRunner:
    """Interface for all job runner implementations. A job runner is responsible for
    the execution logic of multiple jobs.

    Args:
        jobs (List[Job])
    """

    def __init__(self, jobs: List[Job]) -> None:

        self.jobs = jobs

    def execute(self) -> None:
        """Provide a way to execute a list of jobs."""
        raise NotImplementedError()


class JobRunnerStandard(IJobRunner):
    """A simple implementation for executing jobs in sequence."""

    def execute(self) -> None:
        for job in self.jobs:
            job.run()



import json
import logging
from typing import Dict, Iterable, Union

from google.cloud import storage

from renameit.core.exceptions import TargetContainerNotExistError
from renameit.core.models import FileObject
from renameit.core.util import add_slash
from renameit.file_systems.interfaces import IFileSystem


class FileSystemGoogleCloudStorage(IFileSystem):
    """Manage files on google cloud storage.

    Authenticating to google cloud storage uses service account credentials
    and looks for them in the following ordered places:

    - Checks `service_account_json` argument
    - Checks `service_account_info` argument
    - Environment variable called `GOOGLE_APPLICATION_CREDENTIALS`
        that points to json file that contains the credentials

    Args:
        source_bucket (str): Name of bucket that contains the original files.
        target_bucket (str): Name of bucket that should contain the new files.
            If bucket does not already exist an error will be raised, it will not
            be created automatically. Defaults to be the same as source_bucket.
        source_dir (str, optional): Directory path from where to start looking for files.
            If not provided defaults to source_bucket root.
        target_dir (str, optional): Directory path to where should the new files be written.
            If not provided defaults to target_bucket root.
        recursive (bool, optional): Decides whether to look for files recursively.
            Defaults to True.
        service_account_json (str, optional): File path to json file that contains service
            account credentials.
        service_account_info (Union[Dict[str, str], str], optional): Service account
            credentials as dict or a string representing a json object.
    """

    name = "google_cloud_storage"

    def __init__(
        self,
        source_bucket: str,
        target_bucket: str,
        source_dir: str = None,
        target_dir: str = None,
        recursive: bool = False,
        service_account_json: str = None,
        service_account_info: Union[Dict[str, str], str] = None,
    ):
        super().__init__()
        self.source_bucket = source_bucket
        self.target_bucket = target_bucket or source_bucket
        self.source_dir = source_dir
        self.target_dir = target_dir
        self.recursive = recursive
        self.service_account_json = service_account_json
        self.service_account_info = (
            json.loads(service_account_info)
            if type(service_account_info) is str
            else service_account_info
        )

        if self.service_account_json:
            self.storage_client = storage.Client.from_service_account_json(
                self.service_account_json
            )

        elif self.service_account_info:
            self.storage_client = self.storage_client = storage.Client.from_service_account_info(
                self.service_account_info
            )

        else:
            self.storage_client = storage.Client()

        self.source_bucket_client = self.storage_client.bucket(self.source_bucket)
        self.target_bucket_client = self.storage_client.bucket(self.target_bucket)

        # TODO try to create it with the same configs as source_bucket
        # or using some user input.
        if not self.target_bucket_client.exists():
            raise TargetContainerNotExistError(
                f"Target bucket `{self.target_bucket}` does not exist "
                + "or you do not have enough permissions to access it."
            )

    @property
    def source_dir(self):
        return self._source_dir

    @source_dir.setter
    def source_dir(self, value):
        self._source_dir = add_slash(value)

    @property
    def target_dir(self):
        return self._target_dir

    @target_dir.setter
    def target_dir(self, value):
        self._target_dir = add_slash(value)

    def list_objects(self) -> Iterable[FileObject]:
        if self.recursive:
            delimiter = None
        else:
            delimiter = "/"

        blobs = self.storage_client.list_blobs(
            self.source_bucket, prefix=self.source_dir, delimiter=delimiter
        )

        for blob in blobs:
            if not blob.name.endswith("/"):
                yield FileObject(
                    path=blob.name.removeprefix(self.source_dir), modified_date=blob.updated
                )

    def copy_object(self, source_obj: FileObject, target_obj: FileObject) -> None:
        source_blob = self.source_bucket_client.blob(self.source_dir + source_obj.path)
        target_blob = self.source_bucket_client.copy_blob(
            source_blob, self.target_bucket_client, self.target_dir + target_obj.path
        )
        logging.info(
            "Copied: "
            + f"blob {source_blob.name} in bucket {self.source_bucket}"
            + f"to blob {target_blob.name} in bucket {self.target_bucket}"
        )

    def rename_object(self, source_obj: FileObject, target_obj: FileObject) -> None:
        if self.source_bucket == self.target_bucket:
            self._rename_within_same_bucket(source_obj, target_obj)
        else:
            self._rename_across_different_buckets(source_obj, target_obj)

        logging.info(
            "Renamed: "
            + f"blob {self.source_dir + source_obj.path} in bucket {self.source_bucket}"
            + f"to blob {self.target_dir + target_obj.path} in bucket {self.target_bucket}"
        )

    def _rename_within_same_bucket(self, source_obj: FileObject, target_obj: FileObject) -> None:
        source_blob = self.source_bucket_client.blob(self.source_dir + source_obj.path)
        self.source_bucket_client.rename_blob(source_blob, self.target_dir + target_obj.path)

    def _rename_across_different_buckets(
        self, source_obj: FileObject, target_obj: FileObject
    ) -> None:
        self.copy_object(source_obj, target_obj)
        self.source_bucket_client.delete_blob(self.source_dir + source_obj.path)



import logging
import os

__version__ = "0.1.0-alpha.0"

LOGLEVEL = os.environ.get("LOGLEVEL", "WARNING").upper()
logging.basicConfig(
    format="%(asctime)s [%(levelname)s] %(message)s",
    level=LOGLEVEL,
    datefmt="%Y-%m-%d %I:%M:%S %p",
)



from datetime import datetime

import pytest

from renameit.core.models import FileObject
from renameit.handlers.datetime import DateTimeHandler


class TestBasicHandler:
    @pytest.mark.parametrize(
        "dt_value,dt_format,add_as,old_file_object,expected_new_path",
        [
            (
                "modification_date",
                "%Y-%m-%d_",
                "prefix",
                FileObject("file.txt", modified_date=datetime(year=2022, month=1, day=1)),
                "2022-01-01_file.txt",
            ),
            (
                "modification_date",
                "_%Y-%m-%d",
                "suffix",
                FileObject("file.txt", modified_date=datetime(year=2022, month=1, day=1)),
                "file_2022-01-01.txt",
            ),
            (
                "now",
                "%Y-%m-%d_",
                "prefix",
                FileObject("file.txt", modified_date=datetime.utcnow()),
                f"{datetime.utcnow().date()}_file.txt",
            ),
            (
                "now",
                "_%Y-%m-%d",
                "suffix",
                FileObject("file.txt", modified_date=datetime.utcnow()),
                f"file_{datetime.utcnow().date()}.txt",
            ),
            pytest.param(
                "UNSUPPORTED_METHOD",
                "%Y-%m-%d_",
                "prefix",
                FileObject("file.txt"),
                "file.txt",
                marks=pytest.mark.xfail,
            ),
        ],
    )
    def test_get_new_name(self, dt_value, dt_format, add_as, old_file_object, expected_new_path):
        handler = DateTimeHandler(dt_value=dt_value, dt_format=dt_format, add_as=add_as)
        new_file_object = handler.get_new_name(old_file_object)
        assert expected_new_path == new_file_object.path



from typing import Iterable

import boto3

from renameit.core.exceptions import TargetContainerNotExistError
from renameit.core.models import FileObject
from renameit.core.util import add_slash
from renameit.file_systems.interfaces import IFileSystem


class FileSystemAwsS3(IFileSystem):
    """Manage files on aws s3.

    Args:
        source_bucket (str): Name of bucket that contains the original files.
        target_bucket (str, optional): Name of bucket that should contain the new files.
            If bucket does not already exist an error will be raised, it will not
            be created automatically. Defaults to be the same as source_bucket.
        source_dir (str, optional): Directory path from where to start looking for files.
            If not provided defaults to source_bucket root.
        target_dir (str, optional): Directory path to where should the new files be written.
            If not provided defaults to target_bucket root.
        recursive (bool, optional): Decides whether to look for files recursively.
            Defaults to True.
        aws_access_key_id (str, optional): The access key to use when creating
            the s3 resource. If not provided, the credentials configured for the session
            will be used.
        aws_secret_access_key (str, optional): The secret key to use when creating
            the s3 resource. Same semantics as aws_access_key_id above.
        region_name (str, optional):  The name of the region associated with the resource.
            If not provided, it will be obtained from the default boto3 session.
    """

    name = "aws_s3"

    def __init__(
        self,
        source_bucket: str,
        target_bucket: str = None,
        source_dir: str = None,
        target_dir: str = None,
        recursive: bool = False,
        aws_access_key_id: str = None,
        aws_secret_access_key: str = None,
        region_name: str = None,
    ):
        super().__init__()

        self.source_bucket = source_bucket
        self.target_bucket = target_bucket or source_bucket
        self.source_dir = source_dir
        self.target_dir = target_dir
        self.recursive = recursive
        self.aws_access_key_id = aws_access_key_id
        self.aws_secret_access_key = aws_secret_access_key
        self.region_name = region_name or boto3.session.Session().region_name

        self.s3 = boto3.resource(
            "s3",
            aws_access_key_id=self.aws_access_key_id,
            aws_secret_access_key=self.aws_secret_access_key,
            region_name=self.region_name,
        )

        self.s3_client = self.s3.meta.client

        # TODO try to create it with the same configs as source_bucket
        # or using some user input.
        if self.s3.Bucket(self.target_bucket).creation_date is None:
            raise TargetContainerNotExistError(
                f"Target bucket `{self.target_bucket}` does not exist "
                + "or you do not have enough permissions to access it."
            )

    @property
    def source_dir(self):
        return self._source_dir

    @source_dir.setter
    def source_dir(self, value):
        self._source_dir = add_slash(value)

    @property
    def target_dir(self):
        return self._target_dir

    @target_dir.setter
    def target_dir(self, value):
        self._target_dir = add_slash(value)

    def list_objects(self) -> Iterable[FileObject]:
        continuation_token = None
        kwargs = {
            "Bucket": self.source_bucket,
            "Prefix": self.source_dir,
            "StartAfter": self.source_dir,
        }

        # This will group all subdirectories in the response's CommonPrefixes
        # thus, their files will not be included in the reponse's Content.
        if not self.recursive:
            kwargs["Delimiter"] = "/"

        while True:

            if continuation_token:
                kwargs["ContinuationToken"] = continuation_token

            response = self.s3_client.list_objects_v2(**kwargs)

            for obj in response["Contents"]:
                yield FileObject(
                    path=obj["Key"].removeprefix(self.source_dir),
                    modified_date=obj["LastModified"],
                )

            if response["IsTruncated"]:
                continuation_token = response["NextContinuationToken"]
            else:
                break

    def copy_object(self, source_obj: FileObject, target_obj: FileObject) -> None:
        print(self.source_dir + source_obj.path, "->", self.target_dir + target_obj.path)

        copy_source = {"Bucket": self.source_bucket, "Key": self.source_dir + source_obj.path}
        s3_target_bucket = self.s3.Bucket(self.target_bucket)
        s3_target_bucket.copy(copy_source, self.target_dir + target_obj.path)



import json
import os
import pathlib
import tempfile
import uuid

import pytest
from google.cloud import storage

from renameit.core.models import FileObject
from renameit.file_systems.google import FileSystemGoogleCloudStorage

__TEST_ENV_VAR__ = "TEST_GOOGLE_APPLICATION_CREDENTIALS"

FILES = ["file1.txt", "folder/file2.txt", "folder/subfolder/file3.txt"]

RUN_ID = uuid.uuid4().hex
SOURCE_BUCKET = f"renameit-source-pytest-{RUN_ID}"
TARGET_BUCKET = f"renameit-target-pytest-{RUN_ID}"


@pytest.fixture(scope="class")
def google_credentials_json():

    env_var_content = os.environ[__TEST_ENV_VAR__]

    try:
        credentials = pathlib.Path(env_var_content).read_text()
    except FileNotFoundError:

        try:
            credentials = json.loads(env_var_content, strict=False)
        except json.decoder.JSONDecodeError:
            raise ValueError(
                f"Env var {__TEST_ENV_VAR__} provided is neither a "
                + "path to valid and existing json file "
                + "nor a string representation of a json object."
            )

    with tempfile.NamedTemporaryFile(suffix=".json") as _f:
        credentials_json = pathlib.Path(_f.name)
        credentials_json.write_text(credentials)
        yield str(credentials_json)


@pytest.fixture(scope="class", autouse=True)
def google_credentials_env(google_credentials_json):
    os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = google_credentials_json


@pytest.fixture(scope="class")
def storage_client():
    return storage.Client()


@pytest.fixture(scope="class")
def setup(storage_client, request):
    sampe_data = b"Sample data"

    source_bucket = storage_client.bucket(SOURCE_BUCKET)
    target_bucket = storage_client.bucket(TARGET_BUCKET)

    if not source_bucket.exists():
        storage_client.create_bucket(source_bucket)
    if not target_bucket.exists():
        storage_client.create_bucket(target_bucket)

    with tempfile.NamedTemporaryFile() as _f:
        pathlib.Path(_f.name).write_bytes(sampe_data)
        for file_path in FILES:
            blob = source_bucket.blob(file_path)
            blob.upload_from_filename(_f.name)

    def teardown():
        for bucket_name in [SOURCE_BUCKET, TARGET_BUCKET]:
            bucket = storage_client.bucket(bucket_name)
            if bucket.exists():
                bucket.delete(force=True)

    request.addfinalizer(teardown)


@pytest.fixture()
def filesystem(request):
    """Initializes a filesystem with the right params."""

    # Fixed params as there is no need for these to be variable
    params = {
        "source_bucket": SOURCE_BUCKET,
        "target_bucket": TARGET_BUCKET,
    }

    # This allows tests functions or other fixtures to parametarize the
    # creation of filesystem instance.
    # Input params are parsed here if parametrization was defined.
    for arg in {"source_dir", "recursive"}:
        params[arg] = getattr(request, "param", {}).get(arg, None)

    return FileSystemGoogleCloudStorage(**params)


@pytest.mark.skipif(
    not os.environ.get(__TEST_ENV_VAR__),
    reason=f"Env var {__TEST_ENV_VAR__} was not found",
)
@pytest.mark.usefixtures("setup")
class TestFileSystemGoogleStorage:
    """TestFileSystemGoogleStorage."""

    @pytest.mark.parametrize(
        ["filesystem", "expected_list_result"],
        [
            ({"recursive": False}, ["file1.txt"]),
            ({"recursive": True}, FILES),
            ({"source_dir": "folder", "recursive": False}, ["file2.txt"]),
            ({"source_dir": "folder", "recursive": True}, ["file2.txt", "subfolder/file3.txt"]),
        ],
        indirect=["filesystem"],
    )
    def test_list_objects(self, filesystem, expected_list_result):
        file_objects = filesystem.list_objects()
        assert [obj.path for obj in file_objects] == expected_list_result

    @pytest.mark.parametrize(
        ["source_obj", "target_obj"],
        [
            (FileObject("file1.txt"), FileObject("copied/file1.txt")),
            (FileObject("folder/file2.txt"), FileObject("copied/folder/file2.txt")),
        ],
    )
    def test_copy_object(self, storage_client, filesystem, source_obj, target_obj):
        filesystem.copy_object(source_obj, target_obj)
        assert (
            storage_client.bucket(SOURCE_BUCKET)
            .blob(filesystem.source_dir + source_obj.path)
            .exists()
        )
        assert (
            storage_client.bucket(TARGET_BUCKET)
            .blob(filesystem.target_dir + target_obj.path)
            .exists()
        )

    @pytest.mark.parametrize(
        ["source_obj", "target_obj"],
        [
            (FileObject("file1.txt"), FileObject("renamed/file1.txt")),
            (FileObject("folder/file2.txt"), FileObject("renamed/folder/file2.txt")),
        ],
    )
    def test_rename_object(self, storage_client, filesystem, source_obj, target_obj):
        filesystem.rename_object(source_obj, target_obj)
        assert (
            not storage_client.bucket(SOURCE_BUCKET)
            .blob(filesystem.source_dir + source_obj.path)
            .exists()
        )
        assert (
            storage_client.bucket(TARGET_BUCKET)
            .blob(filesystem.target_dir + target_obj.path)
            .exists()
        )



from dataclasses import dataclass
from datetime import datetime
from typing import Optional


@dataclass
class FileObject:
    """This class represents common file metadata."""

    path: str
    modified_date: Optional[datetime] = None

    def __repr__(self) -> str:
        return self.path



"""Module for the base file system interface."""

from importlib import import_module
from typing import Iterable, Optional

from renameit.core.models import FileObject
from renameit.core.util import get_implementations


class IFileSystem:
    """Base interface for implementing file systems.

    Expose specific operations needed to carry out file renaming over a certain type of file system.

    This abstract class leaves the init args open. As initializing different file systems
    have different requirements."""

    # Class name property that will be used to reference the file system type by users.
    name: Optional[str] = None

    def __init__(self) -> None:
        if self.name is None:
            raise AttributeError("`name` is not defined.")

    def list_objects(self) -> Iterable[FileObject]:
        """List files in a container. Should return/yield a generator of FileObject.

        Path of FileObject returned should be subpath relative to its parent path. Meaning, if
        a full path is `/parent_dir/subfoler_1/file1`, return value should be `subfoler_1/file1`.

        This is to prevent any modifications to parent containers outside the scope of this class.
        """
        raise NotImplementedError()

    def rename_object(self, source_obj: FileObject, target_obj: FileObject) -> None:
        """Rename implementation.

        Delete the source object and move it to the target object.

        Args:
            source_obj (FileObject): Original object returned by self.list_objects.
            target_obj (FileObject):New object with modified path provided by a
                renaming handler.
        """
        raise NotImplementedError()

    def copy_object(self, source_obj: FileObject, target_obj: FileObject) -> None:
        """Copy implementation.

        Keep the source object and create a new copy of it as the new target object.

        Args:
            source_obj (FileObject): Original object returned by self.list_objects.
            target_obj (FileObject): New object with modified path provided by a
                renaming handler.
        """
        raise NotImplementedError()


file_systems_map = get_implementations(import_module(__package__), IFileSystem)



import pytest

from renameit.core.models import FileObject
from renameit.handlers.basic import BasicHandler


class TestBasicHandler:
    @pytest.mark.parametrize(
        "method,value,old_file_object,expected_new_path",
        [
            ("add_prefix", "prefix_value_", FileObject("file.txt"), "prefix_value_file.txt"),
            ("add_suffix", "_suffix_value", FileObject("file.txt"), "file_suffix_value.txt"),
            ("remove_prefix", "prefix_value_", FileObject("prefix_value_file.txt"), "file.txt"),
            ("remove_suffix", "_suffix_value", FileObject("file_suffix_value.txt"), "file.txt"),
            pytest.param(
                "UNSUPPORTED_METHOD",
                "value",
                FileObject("file.txt"),
                "file.txt",
                marks=pytest.mark.xfail,
            ),
        ],
    )
    def test_get_new_name(self, method, value, old_file_object, expected_new_path):
        handler = BasicHandler(method=method, value=value)
        new_file_object = handler.get_new_name(old_file_object)
        assert expected_new_path == new_file_object.path



import pathlib

import pytest

from renameit.core.models import FileObject
from renameit.file_systems.local import FileSystemLocal

SOURCE_DIR = "source_files"
TARGET_DIR = "target_files"


@pytest.fixture
def create_sample_files():
    for file in ["file.txt", "subfolder/file.txt"]:
        p = pathlib.Path(SOURCE_DIR) / file
        p.parent.mkdir()
        p.touch()


@pytest.mark.usefixtures("cleandir", "create_sample_files")
class TestFileSystemLocal:
    """TestFileSystemLocal."""

    fs_local = FileSystemLocal(source_dir=SOURCE_DIR, target_dir=TARGET_DIR)

    def test_list_objects(self):
        file_objects = self.fs_local.list_objects()
        assert [obj.path for obj in file_objects] == ["file.txt"]
        self.fs_local.recursive = True
        file_objects = self.fs_local.list_objects()
        assert [obj.path for obj in file_objects] == ["file.txt", "subfolder/file.txt"]

    @pytest.mark.parametrize(
        "source_obj,target_obj",
        [
            (FileObject("file.txt"), FileObject("file_renamed.txt")),
            (FileObject("subfolder/file.txt"), FileObject("subfolder/file_renamed.txt")),
        ],
    )
    def test_rename_object(self, source_obj, target_obj):
        self.fs_local.rename_object(source_obj=source_obj, target_obj=target_obj)
        assert [
            str(p.relative_to(TARGET_DIR)) for p in pathlib.Path(TARGET_DIR).rglob("*.txt")
        ] == [target_obj.path]
        assert source_obj.path not in [
            str(p.relative_to(SOURCE_DIR)) for p in pathlib.Path(SOURCE_DIR).rglob("*.txt")
        ]

    @pytest.mark.parametrize(
        "source_obj,target_obj",
        [
            (FileObject("file.txt"), FileObject("file_copied.txt")),
            (FileObject("subfolder/file.txt"), FileObject("subfolder/file_copied.txt")),
        ],
    )
    def test_copy_object(self, source_obj, target_obj):
        self.fs_local.copy_object(source_obj=source_obj, target_obj=target_obj)
        assert [
            str(p.relative_to(TARGET_DIR)) for p in pathlib.Path(TARGET_DIR).rglob("*.txt")
        ] == [target_obj.path]
        assert source_obj.path in [
            str(p.relative_to(SOURCE_DIR)) for p in pathlib.Path(SOURCE_DIR).rglob("*.txt")
        ]



"""This module contains tests for the main cli entry point.

It uses the local file system by default, this does not mean that the local
file system is fully tested here. Look into tests/file_systems/test_local.py for full tests.

This only tries to test the main program workflow and makes sure core components are working as
expected.
"""


import json
import os
import pathlib
import tempfile

import pytest

from renameit.core.cli import renameit
from renameit.core.config.project import (
    FileSystemConfig,
    JobConfig,
    ProjectConfig,
    RenameHandlerConfig,
)

FILES = ["file1.txt", "folder/file1.txt"]
__FIXED__ = {"PREFIX": "TEST_PREFIX_", "SOURCE_DIR": "source_dir", "TARGET_DIR": "target_dir"}


@pytest.fixture(autouse=True)
def cleandir():
    with tempfile.TemporaryDirectory() as newpath:
        old_cwd = os.getcwd()
        os.chdir(newpath)
        yield
        os.chdir(old_cwd)


@pytest.fixture(autouse=True)
def create_files():
    source_dir = pathlib.Path(__FIXED__["SOURCE_DIR"])
    source_dir.mkdir()
    for file_path in FILES:
        (source_dir / pathlib.Path(file_path).parent).mkdir(parents=True, exist_ok=True)
        (source_dir / file_path).touch()


@pytest.fixture()
def config_file_path(request):
    operation = request.param.get("operation", "rename")
    file_system_config = request.param["file_system_config"]
    rename_handler_config = request.param["rename_handler_config"]

    config_file_path = pathlib.Path("config.json")
    config_file_path.write_text(
        json.dumps(
            ProjectConfig(
                jobs=[
                    JobConfig(
                        name="TEST_JOB",
                        operation=operation,
                        file_system=file_system_config,
                        rename_handler=rename_handler_config,
                    )
                ]
            ).dict(by_alias=True)
        )
    )
    return config_file_path


def _generate_test_parameters():
    return [
        # recursive local rename
        (
            # config_file_path
            {
                "operation": "rename",
                "file_system_config": FileSystemConfig(
                    type="local",
                    args={
                        "source_dir": __FIXED__["SOURCE_DIR"],
                        "target_dir": __FIXED__["TARGET_DIR"],
                        "recursive": True,
                    },
                ),
                "rename_handler_config": RenameHandlerConfig(
                    type="basic", args={"method": "add_prefix", "value": __FIXED__["PREFIX"]}
                ),
            },
            # expected_source_files
            [],
            # expected_target_files
            [
                f"{__FIXED__['TARGET_DIR']}/{__FIXED__['PREFIX']}file1.txt",
                f"{__FIXED__['TARGET_DIR']}/folder/{__FIXED__['PREFIX']}file1.txt",
            ],
        ),
        # recursive local copy
        (
            # config_file_path
            {
                "operation": "copy",
                "file_system_config": FileSystemConfig(
                    type="local",
                    args={
                        "source_dir": __FIXED__["SOURCE_DIR"],
                        "target_dir": __FIXED__["TARGET_DIR"],
                        "recursive": True,
                    },
                ),
                "rename_handler_config": RenameHandlerConfig(
                    type="basic", args={"method": "add_prefix", "value": __FIXED__["PREFIX"]}
                ),
            },
            # expected_source_files
            [__FIXED__["SOURCE_DIR"] + "/" + file for file in FILES],
            # expected_target_files
            [
                f"{__FIXED__['TARGET_DIR']}/{__FIXED__['PREFIX']}file1.txt",
                f"{__FIXED__['TARGET_DIR']}/folder/{__FIXED__['PREFIX']}file1.txt",
            ],
        ),
        # non recursive local rename
        (
            # config_file_path
            {
                "operation": "rename",
                "file_system_config": FileSystemConfig(
                    type="local",
                    args={
                        "source_dir": __FIXED__["SOURCE_DIR"],
                        "target_dir": __FIXED__["TARGET_DIR"],
                        "recursive": False,
                    },
                ),
                "rename_handler_config": RenameHandlerConfig(
                    type="basic", args={"method": "add_prefix", "value": __FIXED__["PREFIX"]}
                ),
            },
            # expected_source_files
            [f"{__FIXED__['SOURCE_DIR']}/folder/file1.txt"],
            # expected_target_files
            [
                f"{__FIXED__['TARGET_DIR']}/{__FIXED__['PREFIX']}file1.txt",
            ],
        ),
        # non recursive local copy
        (
            # config_file_path
            {
                "operation": "copy",
                "file_system_config": FileSystemConfig(
                    type="local",
                    args={
                        "source_dir": __FIXED__["SOURCE_DIR"],
                        "target_dir": __FIXED__["TARGET_DIR"],
                        "recursive": False,
                    },
                ),
                "rename_handler_config": RenameHandlerConfig(
                    type="basic", args={"method": "add_prefix", "value": __FIXED__["PREFIX"]}
                ),
            },
            # expected_source_files
            [__FIXED__["SOURCE_DIR"] + "/" + file for file in FILES],
            # expected_target_files
            [
                f"{__FIXED__['TARGET_DIR']}/{__FIXED__['PREFIX']}file1.txt",
            ],
        ),
        # non recursive local copy - regex handler
        # We are using regex handler here to force a certains use-case
        # which is making sure the new file name stays the same as old
        # and expecting it to be skipped and not cause any problems or
        # some unexpected output.
        (
            # config_file_path
            {
                "operation": "copy",
                "file_system_config": FileSystemConfig(
                    type="local",
                    args={
                        "source_dir": __FIXED__["SOURCE_DIR"],
                        "target_dir": __FIXED__["TARGET_DIR"],
                        "recursive": False,
                    },
                ),
                "rename_handler_config": RenameHandlerConfig(
                    type="regex",
                    # Make sure the args provided result in old_file and new_file to be the same
                    args={"match_pattern": r"(?P<name>\*+)", "replace_pattern": "\\g<name>"},
                ),
            },
            # expected_source_files
            [__FIXED__["SOURCE_DIR"] + "/" + file for file in FILES],
            # expected_target_files
            [],
        ),
    ]


@pytest.mark.parametrize(
    ["config_file_path", "expected_source_files", "expected_target_files"],
    _generate_test_parameters(),
    indirect=["config_file_path"],
)
def test_renameit(config_file_path, expected_source_files, expected_target_files):
    _SOURCE_DIR_ = pathlib.Path(__FIXED__["SOURCE_DIR"])
    _TARGET_DIR_ = pathlib.Path(__FIXED__["TARGET_DIR"])

    renameit(config_path=config_file_path)

    assert list(str(p) for p in _SOURCE_DIR_.glob("**/*.txt")) == expected_source_files
    assert list(str(p) for p in _TARGET_DIR_.glob("**/*.txt")) == expected_target_files



import os
import uuid

import pytest
from azure.storage.filedatalake import DataLakeServiceClient

from renameit.core.models import FileObject
from renameit.file_systems.azure import FileSystemAzureDatalakeStorage

__TEST_ENV_VAR__ = "TEST_AZURE_CONN_STRING"

FILES = ["file1.txt", "folder/file2.txt", "folder/subfolder/file3.txt"]

RUN_ID = uuid.uuid4().hex
SOURCE_CONTAINER = f"renameit-source-pytest-{RUN_ID}"
TARGET_CONTAINER = f"renameit-target-pytest-{RUN_ID}"


@pytest.fixture(scope="class")
def conn_string():
    return os.environ[__TEST_ENV_VAR__]


@pytest.fixture(scope="class")
def service_client(conn_string):
    return DataLakeServiceClient.from_connection_string(conn_string)


@pytest.fixture(scope="class")
def setup(service_client, request):
    sampe_data = b"Sample data"

    fs_client = service_client.get_file_system_client(SOURCE_CONTAINER)
    fs_client.create_file_system()

    for file_path in FILES:
        file_client = fs_client.create_file(file_path)
        file_client.append_data(data=sampe_data, offset=0, length=len(sampe_data))
        file_client.flush_data(len(sampe_data))

    def teardown():
        for file_system_name in [SOURCE_CONTAINER, TARGET_CONTAINER]:
            file_system = service_client.get_file_system_client(file_system_name)
            if file_system.exists():
                file_system.delete_file_system()

    request.addfinalizer(teardown)


@pytest.fixture()
def filesystem(conn_string, request):
    """Initializes a filesystem with the right params."""

    # Fixed params as there is no need for these to be variable
    params = {
        "source_container": SOURCE_CONTAINER,
        "target_container": TARGET_CONTAINER,
        "connection_string": conn_string,
    }

    # This allows tests functions or other fixtures to parametarize the
    # creation of filesystem instance.
    # Input params are parsed here if parametrization was defined.
    for arg in {
        "source_dir",
        "target_dir",
        "recursive",
    }:
        params[arg] = getattr(request, "param", {}).get(arg, None)

    return FileSystemAzureDatalakeStorage(**params)


@pytest.mark.skipif(
    not os.environ.get(__TEST_ENV_VAR__),
    reason=f"Env var {__TEST_ENV_VAR__} was not found",
)
@pytest.mark.usefixtures("setup")
class TestAzureDatalakeStorage:
    """TestAzureDatalakeStorage."""

    @pytest.mark.parametrize(
        ["filesystem", "expected_list_result"],
        [
            ({"recursive": False}, ["file1.txt"]),
            ({"recursive": True}, FILES),
            ({"source_dir": "folder", "recursive": False}, ["file2.txt"]),
            ({"source_dir": "folder", "recursive": True}, ["file2.txt", "subfolder/file3.txt"]),
        ],
        indirect=["filesystem"],
    )
    def test_list_objects(self, filesystem, expected_list_result):
        file_objects = filesystem.list_objects()
        assert [obj.path for obj in file_objects] == expected_list_result

    @pytest.mark.parametrize(
        ["source_obj", "target_obj"],
        [
            (FileObject("file1.txt"), FileObject("renamed/file1.txt")),
            (FileObject("folder/file2.txt"), FileObject("renamed/folder/file2.txt")),
        ],
    )
    def test_rename_object(self, service_client, filesystem, source_obj, target_obj):
        filesystem.rename_object(source_obj, target_obj)
        assert (
            not service_client.get_file_system_client(SOURCE_CONTAINER)
            .get_file_client(source_obj.path)
            .exists()
        )
        assert (
            service_client.get_file_system_client(TARGET_CONTAINER)
            .get_file_client(target_obj.path)
            .exists()
        )



"""
This module acts as the center point for reconciling user input from config files, env vars
and other external configuration services.

Everything related to loading, parsing and rendering configurations can be implemented here.

The result data can be used as input for config classes like the ones in
[project][renameit.core.config.project]
"""

from collections import OrderedDict

import confight


def load_config(user_file_path: str = None, user_dir_path: str = None) -> OrderedDict:
    """Load configurations from files.

    Args:
        user_file_path (str, optional): Path to the base config file. Defaults to None.
        user_dir_path (str, optional): Path to the extension config directory. Defaults to None.

    Returns:
        Single dict with all the loaded config
    """
    kwargs = {}
    if user_file_path is not None:
        kwargs.update({"user_file_path": user_file_path})

    if user_dir_path is not None:
        kwargs.update({"user_dir_path": user_dir_path})

    config = confight.load_user_app("renameit", **kwargs)

    return config



import pytest

from renameit.core.models import FileObject
from renameit.handlers.regex import RegexHandler


class TestRegexHandler:
    @pytest.mark.parametrize(
        "match_pattern,replace_pattern,case_sensitive,old_file_object,expected_new_path",
        [
            (
                r"(?P<year>\d+)/(?P<month>\d+)/(?P<day>\d+)/(?P<file_name>.*)",
                r"year=\g<year>/month=\g<month>/day=\g<day>/\g<file_name>",
                True,
                FileObject("2022/01/01/file.txt"),
                "year=2022/month=01/day=01/file.txt",
            ),
            (
                r"year=(?P<year>\d+)/month=(?P<month>\d+)/day=(?P<day>\d+)/(?P<file_name>.*)",
                r"\g<year>/\g<month>/\g<day>/\g<file_name>",
                True,
                FileObject("year=2022/month=01/day=01/file.txt"),
                "2022/01/01/file.txt",
            ),
            (
                r"year=(?P<year>\d+)/month=(?P<month>\d+)/day=(?P<day>\d+)/(?P<file_name>.*)",
                r"\g<year>/\g<month>/\g<day>/\g<file_name>",
                False,
                FileObject("YEAR=2022/MONTH=01/DAY=01/file.txt"),
                "YEAR=2022/MONTH=01/DAY=01/file.txt",
            ),
        ],
    )
    def test_get_new_name(
        self, match_pattern, replace_pattern, case_sensitive, old_file_object, expected_new_path
    ):
        handler = RegexHandler(
            match_pattern=match_pattern,
            replace_pattern=replace_pattern,
            case_sensitive=case_sensitive,
        )
        new_file_object = handler.get_new_name(old_file_object)
        assert expected_new_path == new_file_object.path



import pathlib
from datetime import datetime
from typing import Optional

from renameit.core.models import FileObject
from renameit.handlers.interfaces import IHandler


class DateTimeHandler(IHandler):
    """Add datetime values to file names.

    Args:
        dt_value (str): Specify where to get date & time values from:

            - `modification_date` uses the file's latest modification datetime
            - `now` uses the current utc datetime

        dt_format (str): How to format the `dt_value` when placing it in the file name.
            Refer to https://strftime.org/ for more info about accepted format directives.
        add_as (str): Where to add `dt_value` in the file name.
            Either "prefix" or "suffix".
    """

    name = "datetime"

    def __init__(self, dt_value: str, dt_format: str, add_as: str):

        super().__init__()

        if dt_value not in ["modification_date", "now"]:
            raise ValueError(
                f"Wrong value `{dt_value}` provided in `dt_value`, "
                + "should be either `now` or `modification_date`."
            )

        if add_as not in ["prefix", "suffix"]:
            raise ValueError(
                f"Wrong value `{add_as}` provided in `add_as`, "
                + "should be either `prefix` or `suffix`."
            )

        self.dt: Optional[datetime] = datetime.utcnow()
        self.dt_value = dt_value
        self.dt_format = dt_format
        self.add_as = add_as

    def get_new_name(self, file_obj: FileObject) -> FileObject:
        path = pathlib.Path(file_obj.path)

        if self.dt_value == "modification_date":
            self.dt = file_obj.modified_date

        if self.dt is not None:
            return FileObject(
                path=str(
                    {
                        "prefix": path.with_name(self.dt.strftime(self.dt_format) + path.name),
                        "suffix": path.with_name(
                            path.stem + self.dt.strftime(self.dt_format) + path.suffix
                        ),
                    }[self.add_as]
                ),
                modified_date=file_obj.modified_date,
            )
        else:
            return file_obj



"""RegexHandler"""

import re

from renameit.core.models import FileObject
from renameit.handlers.interfaces import IHandler


# RegexHandler # noqa
class RegexHandler(IHandler):
    """Manipulate file paths using regular expressions.

    It works by using matching pattern with named groups and then using those groups
    into a replacement string template passed on to the re.sub method.

    Example:
    ```
    match_pattern = r"(?P<year>\\d+)/(?P<month>\\d+)/(?P<day>\\d+)/(?P<file_name>.*)".
    replace_pattern = "year=\\g<year>/month=\\g<month>/day=\\g<day>/\\g<file_name>".
    transforms "2022/01/01/file1.csv" to "year=2022/month=01/day=01/file1.csv".
    ```

    Args:
        match_pattern (str): Regular expressiong with named capturing groups.
        replace_pattern (str): String that contains the named replacement syntax for the
                            caught named groups if there is a match.
        case_sensitive (bool, optional): Applies case sensitive matching. Defaults to True.
    """

    name = "regex"

    def __init__(self, match_pattern: str, replace_pattern: str, case_sensitive: bool = True):

        super().__init__()

        self.match_pattern = match_pattern
        self.replace_pattern = replace_pattern
        self.case_sensitive = case_sensitive

        if case_sensitive:
            self.match_pattern_compiled = re.compile(match_pattern, re.IGNORECASE)
        else:
            self.match_pattern_compiled = re.compile(match_pattern)

    def get_new_name(self, file_obj: FileObject) -> FileObject:
        return FileObject(
            path=self.match_pattern_compiled.sub(self.replace_pattern, file_obj.path),
            modified_date=file_obj.modified_date,
        )



""" Module for the base handler interface."""


from abc import ABC, abstractmethod
from importlib import import_module
from typing import Optional

from renameit.core.models import FileObject
from renameit.core.util import get_implementations


class IHandler(ABC):
    """Base interface for implementing renaming handlers.

    Handler should be initialized with certain criteria for renaming file paths.
    """

    name: Optional[str] = None

    def __init__(self) -> None:
        if self.name is None:
            raise AttributeError("`name` is not defined.")

    @abstractmethod
    def get_new_name(self, file_obj: FileObject) -> FileObject:
        """Change the path of the file_obj according to the provided criteria.

        Args:
            file_obj (FileObject): File object with defined properties.

        Returns:
            FileObject: FileObject instance with modified path.
        """
        raise NotImplementedError()


handlers_map = get_implementations(import_module(__package__), IHandler)
